@@ -1,6 +1,6 @@
-;;; semantic/db-find.el --- Searching through semantic databases.
+;;; semantic/db-find.el --- Searching through semantic databases.  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2000-2018 Free Software Foundation, Inc.
+;; Copyright (C) 2000-2022 Free Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
 ;; Keywords: tags
@@ -209,14 +209,14 @@ This class will cache data derived during various searches.")
   )
 
 (cl-defmethod semanticdb-synchronize ((idx semanticdb-find-search-index)
-				   new-tags)
+				      _new-tags)
   "Synchronize the search index IDX with some NEW-TAGS."
   ;; Reset our parts.
   (semantic-reset idx)
   ;; Notify dependants by clearing their indices.
   (semanticdb-notify-references
    (oref idx table)
-   (lambda (tab me)
+   (lambda (tab _me)
      (semantic-reset (semanticdb-get-table-index tab))))
   )
 
@@ -230,7 +230,7 @@ This class will cache data derived during various searches.")
 	;; Notify dependants by clearing their indices.
 	(semanticdb-notify-references
 	 (oref idx table)
-	 (lambda (tab me)
+	 (lambda (tab _me)
 	   (semantic-reset (semanticdb-get-table-index tab))))
 	)
     ;; Else, not an include, by just a type.
@@ -240,7 +240,7 @@ This class will cache data derived during various searches.")
 	;; Notify dependants by clearing their indices.
 	(semanticdb-notify-references
 	 (oref idx table)
-	 (lambda (tab me)
+	 (lambda (tab _me)
 	   (let ((tab-idx (semanticdb-get-table-index tab)))
 	     ;; Not a full reset?
 	     (when (oref tab-idx type-cache)
@@ -304,7 +304,7 @@ so that it can be called from the idle work handler."
 
 (defun semanticdb-find-translate-path-default (path brutish)
   "Translate PATH into a list of semantic tables.
-If BRUTISH is non nil, return all tables associated with PATH.
+If BRUTISH is non-nil, return all tables associated with PATH.
 Default action as described in `semanticdb-find-translate-path'."
   (if (semanticdb-find-results-p path)
       ;; nil means perform the search over these results.
@@ -315,7 +315,7 @@ Default action as described in `semanticdb-find-translate-path'."
 
 ;;;###autoload
 (define-overloadable-function semanticdb-find-table-for-include (includetag &optional table)
-  "For a single INCLUDETAG found in TABLE, find a `semanticdb-table' object
+  "For a single INCLUDETAG found in TABLE, find a `semanticdb-table' object.
 INCLUDETAG is a semantic TAG of class `include'.
 TABLE is a semanticdb table that identifies where INCLUDETAG came from.
 TABLE is optional if INCLUDETAG has an overlay of :filename attribute."
@@ -362,7 +362,7 @@ Default action as described in `semanticdb-find-translate-path'."
   "Are there any incomplete entries in CACHE?"
   (let ((ans nil))
     (dolist (tab cache)
-      (when (and (semanticdb-table-child-p tab)
+      (when (and (cl-typep tab 'semanticdb-table)
 		 (not (number-or-marker-p (oref tab pointmax))))
 	(setq ans t))
       )
@@ -399,10 +399,10 @@ Default action as described in `semanticdb-find-translate-path'."
   (let ((table (cond ((null path)
 		      semanticdb-current-table)
 		     ((bufferp path)
-		      (semantic-buffer-local-value 'semanticdb-current-table path))
+		      (buffer-local-value 'semanticdb-current-table path))
 		     ((and (stringp path) (file-exists-p path))
 		      (semanticdb-file-table-object path t))
-		     ((semanticdb-abstract-table-child-p path)
+		     ((cl-typep path 'semanticdb-abstract-table)
 		      path)
 		     (t nil))))
     (if table
@@ -426,17 +426,15 @@ Default action as described in `semanticdb-find-translate-path'."
       ;; searchable item, then instead do the regular thing without caching.
       (semanticdb-find-translate-path-includes--internal path))))
 
-(defvar semanticdb-find-lost-includes nil
+(defvar-local semanticdb-find-lost-includes nil
   "Include files that we cannot find associated with this buffer.")
-(make-variable-buffer-local 'semanticdb-find-lost-includes)
 
-(defvar semanticdb-find-scanned-include-tags nil
+(defvar-local semanticdb-find-scanned-include-tags nil
   "All include tags scanned, plus action taken on the tag.
 Each entry is an alist:
   (ACTION . TAG)
-where ACTION is one of 'scanned, 'duplicate, 'lost
+where ACTION is one of `scanned', `duplicate', `lost'
 and TAG is a clone of the include tag that was found.")
-(make-variable-buffer-local 'semanticdb-find-scanned-include-tags)
 
 (defvar semanticdb-implied-include-tags nil
   "Include tags implied for all files of a given mode.
@@ -793,7 +791,8 @@ PREBUTTONTEXT is some text between prefix and the overlay button."
 	 (file (semantic-tag-file-name tag))
 	 (str1 (format "%S %s" mode name))
 	 (str2 (format " : %s" file))
-	 (tip nil))
+	 ;; (tip nil)
+	 )
     (insert prefix prebuttontext str1)
     (setq end (point))
     (insert str2)
@@ -809,7 +808,7 @@ PREBUTTONTEXT is some text between prefix and the overlay button."
     (put-text-property start end 'ddebug (cdr consdata))
     (put-text-property start end 'ddebug-indent(length prefix))
     (put-text-property start end 'ddebug-prefix prefix)
-    (put-text-property start end 'help-echo tip)
+    ;; (put-text-property start end 'help-echo tip)
     (put-text-property start end 'ddebug-function
 		       'data-debug-insert-tag-parts-from-point)
     (insert "\n")
@@ -910,12 +909,12 @@ This query only really tests the first entry in the list that is RESULTP,
 but should be good enough for debugging assertions."
   (and (listp resultp)
        (listp (car resultp))
-       (semanticdb-abstract-table-child-p (car (car resultp)))
+       (cl-typep (car (car resultp)) 'semanticdb-abstract-table)
        (or (semantic-tag-p (car (cdr (car resultp))))
 	   (null (car (cdr (car resultp)))))))
 
 (defun semanticdb-find-result-prin1-to-string (result)
-  "Presuming RESULT satisfies `semanticdb-find-results-p', provide a short PRIN1 output."
+  "If RESULT satisfies `semanticdb-find-results-p', provide a short PRIN1 output."
   (if (< (length result) 2)
       (concat "#<FIND RESULT "
 	      (mapconcat (lambda (a)
@@ -938,7 +937,7 @@ but should be good enough for debugging assertions."
   (and (listp resultp)
        (listp (car resultp))
        (let ((tag-to-test (car-safe (cdr (car resultp)))))
-	 (or (and (semanticdb-abstract-table-child-p (car (car resultp)))
+	 (or (and (cl-typep (car (car resultp)) 'semanticdb-abstract-table)
 		  (or (semantic-tag-p tag-to-test)
 		      (null tag-to-test)))
 	     (and (null (car (car resultp)))
@@ -1011,7 +1010,7 @@ is still made current."
 	  (when norm
 	    ;; The normalized tags can now be found based on that
 	    ;; tags table.
-	    (condition-case foo
+	    (condition-case nil
 		(progn
 		  (semanticdb-set-buffer (car norm))
 		  ;; Now reset ans
@@ -1085,7 +1084,7 @@ Returns result."
   "Log that TABLE has been searched and RESULT was found."
   (when semanticdb-find-log-flag
     (with-current-buffer semanticdb-find-log-buffer-name
-      (insert "Table: " (object-print table)
+      (insert "Table: " (cl-prin1-to-string table)
 	      " Result: " (int-to-string (length result)) " tags"
 	      "\n")
       )
@@ -1245,7 +1244,7 @@ See `semanticdb-find-translate-path' for details on PATH.
 The argument BRUTISH will be set so that searching includes all tables
 in the current project.
 FIND-FILE-MATCH indicates that any time a match is found, the file
-associated wit that tag should be loaded into a buffer."
+associated with that tag should be loaded into a buffer."
   (semanticdb-find-tags-collector
    (lambda (table tags)
      (semanticdb-deep-find-tags-by-name-method table name tags))
@@ -1257,7 +1256,7 @@ See `semanticdb-find-translate-path' for details on PATH.
 The argument BRUTISH will be set so that searching includes all tables
 in the current project.
 FIND-FILE-MATCH indicates that any time a match is found, the file
-associated wit that tag should be loaded into a buffer."
+associated with that tag should be loaded into a buffer."
   (semanticdb-find-tags-collector
    (lambda (table tags)
      (semanticdb-deep-find-tags-for-completion-method table prefix tags))
@@ -1333,6 +1332,9 @@ Returns a table of all matching tags."
       (semantic-find-tags-included (or tags (semanticdb-get-tags table)))
     (semantic-find-tags-by-class class (or tags (semanticdb-get-tags table)))))
 
+(declare-function semantic-find-tags-external-children-of-type
+		  "semantic/find" (type &optional table))
+
 (cl-defmethod semanticdb-find-tags-external-children-of-type-method ((table semanticdb-abstract-table) parent &optional tags)
    "In TABLE, find all occurrences of tags whose parent is the PARENT type.
 Optional argument TAGS is a list of tags to search.
@@ -1340,6 +1342,9 @@ Returns a table of all matching tags."
    (require 'semantic/find)
    (semantic-find-tags-external-children-of-type parent (or tags (semanticdb-get-tags table))))
 
+(declare-function semantic-find-tags-subclasses-of-type
+		  "semantic/find" (type &optional table))
+
 (cl-defmethod semanticdb-find-tags-subclasses-of-type-method ((table semanticdb-abstract-table) parent &optional tags)
    "In TABLE, find all occurrences of tags whose parent is the PARENT type.
 Optional argument TAGS is a list of tags to search.
