@@ -1,6 +1,6 @@
-;;; gnus-ml.el --- Mailing list minor mode for Gnus
+;;; gnus-ml.el --- Mailing list minor mode for Gnus  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2000-2021 Free Software Foundation, Inc.
+;; Copyright (C) 2000-2022 Free Software Foundation, Inc.
 
 ;; Author: Julien Gilles <jgilles@free.fr>
 ;; Keywords: news, mail
@@ -127,7 +127,7 @@ If FORCE is non-nil, replace the old ones."
 	  (t (gnus-message 1 "no list-unsubscribe in this group")))))
 
 (defun gnus-mailing-list-post ()
-  "Post message (really useful ?)"
+  "Post message (really useful ?)."
   (interactive)
   (let ((list-post
 	 (with-current-buffer gnus-original-article-buffer
